@@ -22,12 +22,8 @@ class TestAllocation(base.BaseTestCase):
         super(TestAllocation, self).setUp()
 
         self.rp1 = self.resource_provider_create()
-        self.inv_cpu1 = self.resource_inventory_set(
-            self.rp1['uuid'],
-            'VCPU=4',
-            'VCPU:max_unit=4',
-            'MEMORY_MB=1024',
-            'MEMORY_MB:max_unit=1024')
+        self.resource_inventory_set(
+            self.rp1['uuid'], 'VCPU=4', 'MEMORY_MB=1024')
 
     def test_allocation_show_not_found(self):
         consumer_uuid = str(uuid.uuid4())
@@ -54,17 +50,22 @@ class TestAllocation(base.BaseTestCase):
         self.assertEqual(expected, retrieved_alloc)
 
         # Test that specifying --project-id and --user-id before microversion
-        # 1.8 does not result in an error (they will be ignored). We have
-        # to specify use_json=False because there will be a warning in the
-        # output which can't be json-decoded.
-        output = self.resource_allocation_set(
+        # 1.8 does not result in an error but display a warning.
+        output, warning = self.resource_allocation_set(
             consumer_uuid,
             ['rp={},VCPU=2'.format(self.rp1['uuid']),
              'rp={},MEMORY_MB=512'.format(self.rp1['uuid'])],
-            project_id='fake-project', user_id='fake-user', use_json=False)
+            project_id='fake-project', user_id='fake-user',
+            may_print_to_stderr=True)
+        expected = [
+            {'resource_provider': self.rp1['uuid'],
+             'generation': 3,
+             'resources': {'VCPU': 2, 'MEMORY_MB': 512}}
+        ]
+        self.assertEqual(expected, output)
         self.assertIn(
             '--project-id and --user-id options do not affect allocation for '
-            '--os-placement-api-version less than 1.8', output)
+            '--os-placement-api-version less than 1.8', warning)
 
     def test_allocation_create_empty(self):
         consumer_uuid = str(uuid.uuid4())
@@ -136,12 +137,8 @@ class TestAllocation112(base.BaseTestCase):
         super(TestAllocation112, self).setUp()
 
         self.rp1 = self.resource_provider_create()
-        self.inv_cpu1 = self.resource_inventory_set(
-            self.rp1['uuid'],
-            'VCPU=4',
-            'VCPU:max_unit=4',
-            'MEMORY_MB=1024',
-            'MEMORY_MB:max_unit=1024')
+        self.resource_inventory_set(
+            self.rp1['uuid'], 'VCPU=4', 'MEMORY_MB=1024')
 
     def test_allocation_update(self):
         consumer_uuid = str(uuid.uuid4())
@@ -225,32 +222,49 @@ class TestAllocationUnset112(base.BaseTestCase):
 
     def setUp(self):
         super(TestAllocationUnset112, self).setUp()
-        # Create two providers with inventory.
+        # Create four providers with inventory.
         self.rp1 = self.resource_provider_create()
-        self.inv_cpu1 = self.resource_inventory_set(
-            self.rp1['uuid'],
-            'VCPU=4',
-            'VCPU:max_unit=4',
-            'MEMORY_MB=1024',
-            'MEMORY_MB:max_unit=1024')
         self.rp2 = self.resource_provider_create()
-        self.resource_inventory_set(self.rp2['uuid'], 'VGPU=1')
-        # Create allocations against both providers for the same consumer.
-        self.consumer_uuid = str(uuid.uuid4())
+        self.rp3 = self.resource_provider_create()
+        self.rp4 = self.resource_provider_create()
+
+        self.resource_inventory_set(
+            self.rp1['uuid'], 'VCPU=4', 'MEMORY_MB=1024')
+        self.resource_inventory_set(
+            self.rp2['uuid'], 'VGPU=1')
+        self.resource_inventory_set(
+            self.rp3['uuid'], 'VCPU=4', 'MEMORY_MB=1024', 'VGPU=1')
+        self.resource_inventory_set(
+            self.rp4['uuid'], 'VCPU=4', 'MEMORY_MB=1024', 'VGPU=1')
+
+        self.consumer_uuid1 = str(uuid.uuid4())
+        self.consumer_uuid2 = str(uuid.uuid4())
         self.project_uuid = str(uuid.uuid4())
         self.user_uuid = str(uuid.uuid4())
+
+        # Create allocations against rp1 and rp2 for consumer1.
         self.resource_allocation_set(
-            self.consumer_uuid,
+            self.consumer_uuid1,
             ['rp={},VCPU=2'.format(self.rp1['uuid']),
              'rp={},MEMORY_MB=512'.format(self.rp1['uuid']),
              'rp={},VGPU=1'.format(self.rp2['uuid'])],
             project_id=self.project_uuid, user_id=self.user_uuid)
 
+        # Create allocations against rp3 and rp4 for consumer1.
+        self.resource_allocation_set(
+            self.consumer_uuid2,
+            ['rp={},VCPU=1'.format(self.rp3['uuid']),
+             'rp={},MEMORY_MB=256'.format(self.rp3['uuid']),
+             'rp={},VGPU=1'.format(self.rp3['uuid']),
+             'rp={},VCPU=1'.format(self.rp4['uuid']),
+             'rp={},MEMORY_MB=256'.format(self.rp4['uuid'])],
+            project_id=self.project_uuid, user_id=self.user_uuid)
+
     def test_allocation_unset_one_provider(self):
         """Tests removing allocations for one specific provider."""
         # Remove the allocation for rp1.
         updated_allocs = self.resource_allocation_unset(
-            self.consumer_uuid, provider=self.rp1['uuid'])
+            self.consumer_uuid1, provider=self.rp1['uuid'])
         expected = [
             {'resource_provider': self.rp2['uuid'],
              'generation': 3,
@@ -260,12 +274,62 @@ class TestAllocationUnset112(base.BaseTestCase):
         ]
         self.assertEqual(expected, updated_allocs)
 
+    def test_allocation_unset_one_resource_class(self):
+        """Tests removing allocations for resource classes."""
+        updated_allocs = self.resource_allocation_unset(
+            self.consumer_uuid2, resource_class=['MEMORY_MB'])
+        expected = [
+            {'resource_provider': self.rp3['uuid'],
+             'generation': 3,
+             'project_id': self.project_uuid,
+             'user_id': self.user_uuid,
+             'resources': {'VCPU': 1, 'VGPU': 1}},
+            {'resource_provider': self.rp4['uuid'],
+             'generation': 3,
+             'project_id': self.project_uuid,
+             'user_id': self.user_uuid,
+             'resources': {'VCPU': 1}}
+        ]
+        self.assertEqual(expected, updated_allocs)
+
+    def test_allocation_unset_resource_classes(self):
+        """Tests removing allocations for resource classes."""
+        updated_allocs = self.resource_allocation_unset(
+            self.consumer_uuid2, resource_class=['VCPU', 'MEMORY_MB'])
+        expected = [
+            {'resource_provider': self.rp3['uuid'],
+             'generation': 3,
+             'project_id': self.project_uuid,
+             'user_id': self.user_uuid,
+             'resources': {'VGPU': 1}}
+        ]
+        self.assertEqual(expected, updated_allocs)
+
+    def test_allocation_unset_provider_and_rc(self):
+        """Tests removing allocations of resource classes for a provider ."""
+        updated_allocs = self.resource_allocation_unset(
+            self.consumer_uuid2, provider=self.rp3['uuid'],
+            resource_class=['VCPU', 'MEMORY_MB'])
+        expected = [
+            {'resource_provider': self.rp3['uuid'],
+             'generation': 3,
+             'project_id': self.project_uuid,
+             'user_id': self.user_uuid,
+             'resources': {'VGPU': 1}},
+            {'resource_provider': self.rp4['uuid'],
+             'generation': 3,
+             'project_id': self.project_uuid,
+             'user_id': self.user_uuid,
+             'resources': {'VCPU': 1, 'MEMORY_MB': 256}},
+        ]
+        self.assertEqual(expected, updated_allocs)
+
     def test_allocation_unset_remove_all_providers(self):
         """Tests removing all allocations by omitting the --provider option."""
         # For this test pass use_json=False to make sure we get nothing back
         # in the output since there are no more allocations.
         updated_allocs = self.resource_allocation_unset(
-            self.consumer_uuid, use_json=False)
+            self.consumer_uuid1, use_json=False)
         self.assertEqual('', updated_allocs.strip())
 
 
